/* Programme démonstrateur pour un dialogue série par l'intermédiaire des deux lignes
   filaires TX et RX des cartes Arduino NANO. Programme ESCLAVE qui en version complète
   sera téléchagé sur la carte NANO de la sonde JEKERT.
   
 La procédure Attendre_une_chaine_Esclave() est différente de celle du Maître car surveille
 comme sentinelle l'arrivée d'un caractère '*'. Ce protocole suppose que le Maître termine
 toutes ses consignes par le caractère de fin de transmission '*'.
 
 UTILISATION DU DEMONSTRATEUR :
 ------------------------------
 Ce programme est intimement lié au programme P18_Maitre_minimal. Les deux circuits sont reliés
 en mode filaire croisé sur les lignes TX et RX. Ce programme se contente d'attendre une consigne
 terminée par la sentinelle '*'. Chaque attente d'une chaîne sur la ligne RX est précédée
 d'un BIP permettant de surveiller les alternats et le non blocage d'un dialogue.
 Si la consigne reçue est "a*" il allume la LED et retourne "!OK!" en ACR. Si la consigne reçue est
 "l*" le programme éteint la LED et retourne "!ERR!". Ce n'est pas réellement une erreur, c'est pour
 que le Maître puisse faire la distinction entre les deux accusés de réception.
 
 DIFFERENTS BRANCHEMENTS :
 D0 : RX > Ligne série de dialogue va à TX du Maître.
 D1 : TX > Ligne série de dialogue va à RX du Maître.
 D2 : BUZZER.
 D4 : Disjoncteur.
 D7 : LED témoin.
                                                             PGM               RAM dynamique 
 Programme démonstrateur pour Esclave minimal :       5858 Octets (19%)    240 Octets (11%)      */
 
//--------------------- Déclarations pour le matériel -----------------------
//-------------- Définition des broches de sorties Binaires -----------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.
#define LED_disjoncteur 4
#define LED_temoin 7

//------------------------ Constantes du programme --------------------------
#define LGR_chaine 5 // Longueur maximale de la chaine de dialogue.
// (Dans ce démonstrateur la plus longue chaîne reçue sera "!ERR!".)

//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.

boolean Sentinelle_detectee;  // Délimiteur de fin détecté.
char Caractere;
byte Compteur_caracteres;
String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

void setup()  {
  Serial.begin(115200);
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  pinMode(LED_temoin, OUTPUT); pinMode(LED_disjoncteur, OUTPUT); digitalWrite(LED_disjoncteur,HIGH);}

void loop() {
  //-------------- Traite une consigne arrivée sur USB ---------------
    if (Serial.available()) {
        Attendre_une_chaine_Esclave();
        if (Chaine_Memorisee == "a*")
           {digitalWrite(LED_temoin,HIGH); Serial.print("!OK!");}
        if (Chaine_Memorisee == "l*")
           {digitalWrite(LED_temoin,LOW); Serial.print("!ERR!");} }
  }
  
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
        
void Attendre_une_chaine_Esclave() {// Cette procédure attend une chaine sur la ligne USB.
//-------------- Définition des broches de sorties Binaires -----------------
  BIP();
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres=0;
  while (!Sentinelle_detectee) {
        if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
        // Puis concaténer ce caractère y compris la sentinelle :
        if (Compteur_caracteres < LGR_chaine) 
            {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}
        // Si sentinelle détectée, informer la boucle de réception :
        Sentinelle_detectee = (Caractere == '*'); }} }    
