/* Programme démonstrateur pour un dialogue série rapide et en boucle automatique sur des deux lignes filaires
   TX et RX des cartes Arduino NANO. Directement dérivé de P18 il envoie en permanence des consignes alternées
   "a*" et "l*" sur TX puis passe à l'écoute et inverse la LED témoin à chaque alternat.
   Tous les 1000 alternats il génère un BIP pour information sonore.
 
  UTILISATION DU DEMONSTRATEUR :
  ------------------------------
 Relier la ligne TX/RX à la carte Arduino de la sonde qui contient le programme P30_Programme_COMPLET_T5.
 Faire un RESET sur l'Esclave puis sur le Maître. Immédiatement le dialogue en rafale doit s'amorcer.
 
 DIFFERENTS BRANCHEMENTS :
 A0 : Clavier provisoire à deux boutons câblés en analogique.
 D0 :  RX > Ligne série de dialogue va à TX du Maître.
 D1 :  TX > Ligne série de dialogue va à RX du Maître.
 D6 : BUZZER.
 D4 : LED témoin du dialogue.
 
                                                             PGM               RAM dynamique 
 Programme démonstrateur Maître minimal :               5894 Octets (19%)     233 Octets (11%)   */
   
//--------------------- Déclarations pour le matériel -----------------------

//-------------- Définition des broches de sorties Binaires -----------------
#define BUZZER 6 // Le bruiteur est piloté par la sortie binaire D6.
#define LED_Dialogue 4

//------------------------ Constantes du programme --------------------------
#define LGR_chaine 15 // Longueur maximale de la chaine de dialogue.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.
boolean Premier_delimiteur_OK; // La sentinelle sera le deuxième délimiteur '!'.
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.
char Caractere;
byte Compteur_caracteres;
int Nb_alternats;
String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

void setup()  {
  Serial.begin(115200); Nb_alternats = 0;
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  pinMode(LED_Dialogue, OUTPUT); }

void loop() {
  Serial.print("a*"); Traiter_ACR(); delay(3);
  Serial.print("l*"); Traiter_ACR(); delay(3);
  Nb_alternats++;
  if (Nb_alternats == 1000) {BIP(); Nb_alternats = 0;}}
      
void Traiter_ACR() {
  Attendre_une_chaine_sur_le_Maitre(); digitalWrite(LED_Dialogue,!digitalRead(LED_Dialogue));}
  
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
  
void Attendre_une_chaine_sur_le_Maitre() {// Cette procédure attend une chaine sur la ligne USB.
//-------------- Définition des broches de sorties Binaires -----------------
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Premier_delimiteur_OK = false; // Premier '!' non encore reçu.
  Compteur_caracteres=0;
  while (!Sentinelle_detectee) {
        if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
        // Puis concaténer ce caractère y compris la sentinelle :
        if (Compteur_caracteres < LGR_chaine) 
            {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}
        // Si sentinelle détectée, informer la boucle de réception :
        Sentinelle_detectee = ((Caractere == '!') && (Premier_delimiteur_OK));
        Premier_delimiteur_OK = true; }}} // On saute le premier '!'.
