/* Programme Démonstrateur pour mettre au point le PUPITRE.
   Programme à associer avec P22A_Demonstrateur_Sonde.ino
   
 UTILISATION DU PROGRAMME :
 --------------------------
 1) Établir la liaison TX/RX avec l'inverseur,
 2) Faire un RESET sur la Raquette,
 3) Valider l'éditeur de P21A_Demonstrateur_Raquette.ino sur le P.C.
 4) Activer le Moniteur de l'IDE. (Le P.C. affichera les ACR de la sonde.
 5) Cliquer à convenance sur toutes les touches du clavier.
    - Les LEDs autres que celles de S13 à S15 sont pilotées.
    - Si une LED ne peut pas encore être pilotée l'ACR est son Code. Comme l'ACR est
      différent de "OK*" la raquette considère que c'est une ERREUR et génère un BIP.
    - La touche S16 indique en ACR la version du logiciel de la Sonde.
 (Le BIP sonose sera généré avec les touches S9, S10,S13, S14, S15 et S16) 
   
 Ce que fait ce démonstrateur :
 • Il met en place les routines de transmission d'une Consigne sur TX et de réception ACR sur RX.
 • Il met en place la procédure LOOP de base et la LED clignotante.
 • Il met en place la procédure de gestion d'un CLAVIER à 16 touches.
 • Il transmet les 16 Consignes de 24 à 39 qui positionnent les booléens d'état de la sonde.
 • Effectue la réception de l'ACR et vérifie "OK*" ou génère un BIP sonore.

                                                             PGM            RAM dynamique 
Lecture ACR et clavier multiplexé installés :          6632 Octets (21%)   235 Octets (11%)
   
//--------------------- Déclarations pour le matériel ----------------------- */
//--------------- Définition des broches d'Entrées Analogiques --------------
#define Mesure_U_clavier 6 // Entrée analogique A6 utilisée pour le clavier.
//--------------- Définition des broches d'Entrées BINAIRES -----------------
//-------------- Définition des broches de sorties Binaires -----------------
#define Disjoncteur 4 // Masse des LEDs pour le mode Sommeil : Sortie binaire D4.
#define BUZZER 6
#define C1 7  // Colonne C1 sur la sortie D7.
#define C2 8  // Colonne C1 sur la sortie D8.
#define C3 9  // Colonne C1 sur la sortie D9.
#define C4 10 // Colonne C1 sur la sortie D10.
#define LED_Loop_active 13 // D13 utilisée pour la LED clignotante "Boucle de base".
//------------------------ Constantes du programme --------------------------
#define LGR_chaine 14 //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.

char Caractere;
byte BP; // Bouton du claviar activé.
byte Code_Consigne;
byte Compteur_caracteres;

int CNA; // Mesure analogique retournée par le CNA.

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

void setup()  {
  Serial.begin(115200);
  pinMode(C1,OUTPUT); pinMode(C2,OUTPUT); pinMode(C3,OUTPUT); pinMode(C4,OUTPUT);
  digitalWrite(C1,LOW); digitalWrite(C2,LOW); digitalWrite(C3,LOW); digitalWrite(C4,LOW); 
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  }

void loop() {
  //-------- Faire clignoter la LED de la boucle de base active ----------
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + 100; // Réarme pour 0,1S ou 0,03S.
      digitalWrite(LED_Loop_active,!digitalRead(LED_Loop_active));} // Inverser l'état de la LED.
  //------------------------ Traiter le clavier --------------------------
  Tester_le_clavier();
  if (BP > 0) {
      Traite_un_clic_clavier(); Attendre_une_chaine_sur_le_Maitre();
      if (Chaine_Memorisee != "OK*") BIP();}
  }
  
void Traite_un_clic_clavier() {
  switch (BP) {
     case 1 : {Envoyer_Consigne(25); break;} // S1 Disjoncter.
     case 2 : {Envoyer_Consigne(26); break;} // S2 Rétablir les éclairages.
     case 3 : {Envoyer_Consigne(27); break;} // S3 Allumer les Phares.
     case 4 : {Envoyer_Consigne(28); break;} // S4 Éteindre les Phares.
     case 5 : {Envoyer_Consigne(29); break;} // S5 Allumer le LASER.
     case 6 : {Envoyer_Consigne(30); break;} // S6 Éteindre le LASER.    
     case 7 : {Envoyer_Consigne(31); break;} // S7 Figer les Moteurs.
     case 8 : {Envoyer_Consigne(32); break;} // S8 Rétablir les Moteurs.
     case 9 : {Envoyer_Consigne(33); break;} // S9 Mouvements coordonné Rapides.
     case 10 : {Envoyer_Consigne(34); break;} // S10 Mouvements coordonnés  Lents.
     case 11 : {Envoyer_Consigne(35); break;} // S11 Stabilisation Gyroscopique active.
     case 12 : {Envoyer_Consigne(36); break;} // S12 Stabilisation Gyroscopique suspendue.
     case 13 : {Envoyer_Consigne(37); break;} // S13 Débuter un apprentissage.
     case 14 : {Envoyer_Consigne(38); break;} // S14 Terminer un apprentissage.
     case 15: {Envoyer_Consigne(39); break;} // S15 Terminer un mouvement de torsion.
     case 16 : {Envoyer_Consigne(24); break;}}} // S16 Retourne la version de la Sonde.

//------------------------- Gestion du clavier à 16 touches --------------------------
void Tester_le_clavier() {
   BP = 0; // Indique une non utilisation du clavier.
  Teste_Colonne(C1); Teste_Colonne(C2); Teste_Colonne(C3); Teste_Colonne(C4);}

void Teste_Colonne(byte Colonne) {
  digitalWrite(Colonne, HIGH); // Passe la colonne au +5Vcc.
  CNA = analogRead(Mesure_U_clavier);
  if (CNA > 90) // Codage du BP sollicité.
     {BP = Colonne + 6; if (CNA > 286) BP = BP - 4; if (CNA > 511) BP = BP - 4; if (CNA > 718) BP = BP - 4;
      while (analogRead(Mesure_U_clavier) > 90) delay (5); delay (10);} // Attendre le relâcher.
  digitalWrite(Colonne, LOW);} // Repasse la colonne à zéro volts.
 
//------------------------------------------------------------------------------------
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}

void Attendre_une_chaine_sur_le_Maitre() {// Cette procédure attend une chaine sur la ligne USB.
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres = 0;
  while (!Sentinelle_detectee) {
    if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
    // Puis concaténer ce caractère y compris la sentinelle :
    if (Compteur_caracteres < LGR_chaine) 
       {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}
    // Si sentinelle détectée, informer la boucle de réception :
    Sentinelle_detectee = (Caractere == '*');}}}    
         
void Envoyer_Consigne(byte CODE) {Serial.print(CODE); Serial.print('*');}
