/* Programme Démonstrateur pour mettre au point le PUPITRE.
   Programme à associer avec P22B_Demonstrateur_Sonde.ino
   
   UTILISATION DU PROGRAMME :
 ----------------------------
 1) Établir la liaison TX/RX avec l'inverseur,
 2) Faire un RESET sur la Raquette,
 3) Valider l'éditeur de P22B_Demonstrateur_Sonde.ino sur le P.C.
 4) Activer le Moniteur de l'IDE. (Le P.C. affichera les ACR de la sonde, donc les n°ERR et la version.
 5) Cliquer à convenance sur toutes les touches du clavier.
    Si ERR ou version du PGM un BIP est déclenché.
 6) Cliquer sur le bouton central du codeur incrémental se contente de générer un BIP.
 7) Tourner à convenance le codeur incrémental. Il n'y aura un effet sur le répétiteur, sur le LASER et
    sur les Phares que si une CIBLE est préalablement désignée avec S13 ou S14. Les débordements
    dans un sens ou dans l'autre génèrent des erreurs n°1 ou n°2.
 Pour tester la LED rouge BUZZER sur la sonde, S9 et S10 de Rapide et Lent alternent son état.
   
 Ce que fait ce démonstrateur :
 • Il installe le codeur rotatif et le B.P. central
 • Le B.P. central se contente de faire un BIP.
 • Il transmet les Consignes 24 à 36 de P21A_Demonstrateur_Raquette.ino.
 • Fonction du sens du codeur incrémental il envoie la consigne 40 ou 41.
 • Traite la sélection de la CIBLE avec les codes 42 ou 43.
 • Le BIP sonore est complété d'une alerte lumineuse sur la LED rouge D4.
 • Si l'ACR est différent de "OK*" génère un BIP sonore.

                                                             PGM            RAM dynamique 
Démonstrateur avant modifications :                    6632 Octets (21%)   235 Octets (11%)
Gestion due codeur rotatif et du B.P. central          6812 Octets (22%)   238 Octets (11%) +180 Octets
   
//--------------------- Déclarations pour le matériel ----------------------- */
// Déclarations pour le capteur codeur rotatif KY-40 :
#define Codeur_A 2    // D2 : INT 1 sur broche 5 du microcontrôleur. (CLK)
#define Codeur_B 3    // D3 : INT 0 sur broche 4 du microcontrôleur. (DT)
#define BP_central 21 // A7 pour lire le bouton central.
//--------------- Définition des broches d'Entrées Analogiques --------------
#define Mesure_U_clavier 6 // Entrée analogique A6 utilisée pour le clavier.
#define BP_central 21 // A7 pour lire le bouton central.
//-------------- Définition des broches de sorties Binaires -----------------
#define LED_systeme 4 // LED rouge système.
#define BUZZER 6
#define C1 7  // Colonne C1 sur la sortie D7.
#define C2 8  // Colonne C1 sur la sortie D8.
#define C3 9  // Colonne C1 sur la sortie D9.
#define C4 10 // Colonne C1 sur la sortie D10.
#define LED_Loop_active 13 // D13 utilisée pour la LED clignotante "Boucle de base".
//------------------------ Constantes du programme --------------------------
#define LGR_chaine 14
//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.
boolean Sens_positif;   // Sens de rotation.
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.

byte BP; // Bouton du claviar activé.
byte Code_Consigne;
byte Compteur_caracteres;
byte Etat_B;
byte Transition;
char Caractere;

int CNA; // Mesure analogique retournée par le CNA.

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

void setup()  {
  Serial.begin(115200);
  //------------------------- Initialisations des variables ----------------------------
  Transition = HIGH; Etat_B = LOW;
  //------------------------------ Initialisations des E/S -----------------------------
  pinMode(C1,OUTPUT); pinMode(C2,OUTPUT); pinMode(C3,OUTPUT); pinMode(C4,OUTPUT);
  digitalWrite(C1,LOW); digitalWrite(C2,LOW); digitalWrite(C3,LOW); digitalWrite(C4,LOW);
  pinMode(Codeur_A, INPUT); pinMode(Codeur_B, INPUT); 
  pinMode(LED_systeme,OUTPUT); digitalWrite(LED_systeme,LOW);
  Chaine_Memorisee.reserve(LGR_chaine);} // Reserve LGR_chaine octets pour la chaine.

void loop() {
  //--------------- Faire clignoter la LED de la boucle de base active -----------------
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + 100; // Réarme pour 0,1S ou 0,03S.
      digitalWrite(LED_Loop_active,!digitalRead(LED_Loop_active));} // Inverser l'état de la LED.
  //------------------------------- Traiter le clavier ---------------------------------
  Tester_le_clavier();
  if (BP > 0) {
      Traite_un_clic_clavier(); Attendre_une_chaine_sur_le_Maitre();
      if (Chaine_Memorisee != "OK*") BIP();}
  //----------------------------- Traite le B.P. central -------------------------------
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     BIP();}
  //------------------------------ Traite la rotation ----------------------------------
  Etat_B = digitalRead(Codeur_B);  
  if ((Transition == LOW) && (Etat_B == HIGH)) {    
       if (digitalRead(Codeur_A) == LOW) Sens_positif = false;
       else Sens_positif = true;
       if (Sens_positif) Envoyer_Consigne(40); else Envoyer_Consigne(41);
       Attendre_une_chaine_sur_le_Maitre(); if (Chaine_Memorisee != "OK*") BIP();}
  Transition = Etat_B;}
  
void Traite_un_clic_clavier() {
  switch (BP) {
     case 1 : {Envoyer_Consigne(25); break;} // S1 Disjoncte les LEDs.
     case 2 : {Envoyer_Consigne(26); break;} // S2 Rétablit les LEDs.
     case 3 : {Envoyer_Consigne(27); break;} // S3 Allumer les Phares.
     case 4 : {Envoyer_Consigne(28); break;} // S4 Éteindre les Phares.
     case 5 : {Envoyer_Consigne(29); break;} // S5 Allumer le LASER.
     case 6 : {Envoyer_Consigne(30); break;} // S6 Éteindre le LASER.    
     case 7 : {Envoyer_Consigne(31); break;} // S7 Figer les moteurs.
     case 8 : {Envoyer_Consigne(32); break;} // S8 Rétablir les moteurs.
     case 9 : {Envoyer_Consigne(33); break;} // S9 Mvt de coordination rapides.
     case 10 : {Envoyer_Consigne(34); break;} // S10 Mvt de coordination lents.
     case 11 : {Envoyer_Consigne(35); break;} // S11 Début de stabilisation gyroscopique.
     case 12 : {Envoyer_Consigne(36); break;} // S12 Fin de stabilisation gyroscopique.
     case 13 : {Envoyer_Consigne(42); break;} // S13 Le codeur incrémental agit sur les LASER.
     case 14 : {Envoyer_Consigne(43); break;} // S14 Le codeur incrémental agit sur les Phares.
     case 15 : {Envoyer_Consigne(39); break;} // S15 Fin de la torsion active.
     case 16 : {Envoyer_Consigne(24); break;}}} // S16 Indique la version du programme.

//--------------------------------- Gestion du clavier à 16 touches --------------------------------
void Tester_le_clavier() {
   BP = 0; // Indique une non utilisation du clavier.
  Teste_Colonne(C1); Teste_Colonne(C2); Teste_Colonne(C3); Teste_Colonne(C4);}

void Teste_Colonne(byte Colonne) {
  digitalWrite(Colonne, HIGH); // Passe la colonne au +5Vcc.
  CNA = analogRead(Mesure_U_clavier);
  if (CNA > 90) // Codage du BP sollicité.
     {BP = Colonne + 6; if (CNA > 286) BP = BP - 4; if (CNA > 511) BP = BP - 4; if (CNA > 718) BP = BP - 4;
      while (analogRead(Mesure_U_clavier) > 90) delay (5); delay (10);} // Attendre le relâcher.
  digitalWrite(Colonne, LOW);} // Repasse la colonne à zéro volts.

//--------------------------------------------------------------------------------------------------
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); digitalWrite(LED_systeme,HIGH);
  delay(100); noTone(BUZZER); digitalWrite(LED_systeme,LOW); pinMode(BUZZER, INPUT);}

void Attendre_une_chaine_sur_le_Maitre() {// Cette procédure attend une chaine sur la ligne USB.
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres = 0;
  while (!Sentinelle_detectee) {
    if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
    // Puis concaténer ce caractère y compris la sentinelle :
    if (Compteur_caracteres < LGR_chaine) 
       {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}
    // Si sentinelle détectée, informer la boucle de réception :
    Sentinelle_detectee = (Caractere == '*');}}}    
         
void Envoyer_Consigne(byte CODE) {Serial.print(CODE); Serial.print('*');}
