/* Programme Démonstrateur pour animer la Sonde JEKERT.
   Programme à associer avec P21A_Demonstrateur_Raquette.ino
   
 UTILISATION DU PROGRAMME :
 --------------------------
 1) Établir la liaison TX/RX avec l'inverseur,
 2) Faire un RESET sur la Raquette,
 3) Valider l'éditeur de P21A_Demonstrateur_Raquette.ino sur le P.C.
 4) Activer le Moniteur de l'IDE. (Le P.C. affichera les ACR de la sonde.
 5) Cliquer à convenance sur toutes les touches du clavier.
    - Les LEDs autres que celles de S13 à S15 sont pilotées.
    - Si une LED ne peut pas encore être pilotée l'ACR est son Code. Comme l'ACR est
      différent de "OK*" la raquette considère que c'est une ERREUR et génère un BIP.
    - La touche S16 indique en ACR la version du logiciel de la Sonde. 
  (Le BIP sonose sera généré avec les touches S9, S10,S13, S14, S15 et S16)
  
 Ce que fait ce démonstrateur :
 • Il met en place les routines de réception d'une Consigne et de transmission ACR sur TX.
 • Il intègre l'analyseur syntaxique de base qui extrait le n° de la Consigne dans NOMBRE.
 • Il met en place la procédure LOOP de base et la LED clignotante.
 • Il traite les 16 consignes de 24 à 39 qui positionnent les booléens d'état de la sonde.

                                                             PGM               RAM dynamique 
 Programme démonstrateur Esclave minimal :              4518 Octets (14%)    222 Octets (10%)
 Intégration de 16 fonctions bascules d'état Sonde      5598 Octets (18%)    282 Octets (13%)
*/
   
//--------------------- Déclarations pour le matériel -----------------------
//--------------- Définition des broches d'Entrées Analogiques --------------
//--------------- Définition des broches d'Entrées BINAIRES -----------------
//------------------ Sorties sur broches "Analogiques" ----------------------
#define LED_Torsion_active 16 // Entrée analogique A2 utilisée comme Sortie Binaire.
#define LED_Loop_active 17 // Entrée analogique A3 utilisée comme Sortie Binaire.
//-------------- Définition des broches de sorties Binaires -----------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.
#define Disjoncteur 4 // Masse des LEDs pour le mode Sommeil : Sortie binaire D4.
#define LED_Moteurs_OFF 7 // Pilotage LED témoin Moteurs figés.
#define Potentiometre_suspendu 8 // Pilotage LED témoin Potentiomètre Neutralisé.
//-------------- Broches de sorties Binaires utilisées en PWM ---------------
#define Pilotage_eclairage 5 // Pilotage des deux LEDs phares de la sonde : Sortie binaire D5.
#define LASER 6 // Sortie binaire D6 pour piloter le LASER en PWM.
//------------------------ Constantes du programme --------------------------
#define Version_PGM "> P22-A <" // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LGR_chaine 4 // Longueur maximale de la chaine de dialogue : Quatre en réception.
#define LGR_ACR 13 // Longueur maximale de la chaine d'ACR sur TX. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.
boolean Apprentissage; // Mode apprentissage en cours.
boolean LASER_actif; // Allumer ou éteindre le LASER.
boolean Moteurs_ON; // Si false tous les moteurs sont figés.
boolean Phares_actifs; // Allumer ou éteindre les phares.
boolean Rapide; // Les mouvements coordonnés seront rapides.
boolean Repondre;
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.
boolean Stabilisation_gyroscopique;
boolean Torsion_Active; // Les corrections en torsion sont actives.

char Caractere;
byte Compteur_caracteres;
byte LSB; // Utilisé par le capteur magnétique.
byte Niveau_eclairage_LASER; // Pilotage analogique du LASER.
byte Niveau_eclairage_Phares; // Pilotage analogique des deux phares.
byte num_ERR;

int  NOMBRE; // Contiendra la transformée numérique d'une chaine.

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.
String Texte_ACR; // Tampon mémoire pour la chaine sortante.

void setup()  {
  Serial.begin(115200); Repondre = false;
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  Chaine_Memorisee.reserve(LGR_ACR); // Reserve LGR_chaine octets pour la chaine.
  //--------------------------- Initialiser les booléens ---------------------------
  Phares_actifs = false; LASER_actif = false;  Stabilisation_gyroscopique = false;
  Rapide = true; Torsion_Active = false; 
  Moteurs_ON = true; // À enlever quand le module inertiel sera introduit @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  //--------------------------- Initialiser les sorties ----------------------------
  Niveau_eclairage_LASER = 254; Niveau_eclairage_Phares = 254; // Provisoire @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  pinMode(Disjoncteur,OUTPUT); digitalWrite(Disjoncteur,HIGH); // Transistor saturé. (Masse active.)
  pinMode(LED_Loop_active,OUTPUT); pinMode(LED_Moteurs_OFF,OUTPUT); digitalWrite(LED_Moteurs_OFF,LOW);
  pinMode(Potentiometre_suspendu,OUTPUT); digitalWrite(Potentiometre_suspendu,LOW);
  pinMode(LED_Torsion_active,OUTPUT); digitalWrite(LED_Torsion_active,LOW);
  //--------------------------------------------------------------------------------
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  }

void loop() {
  //--------------------- Traite une Consigne arrivée sur USB ----------------------
  if (Serial.available()) Attendre_une_chaine_sur_le_Maitre();
  //-------------------------- Traitement de la Consignee --------------------------
  if (Chaine_Memorisee.length() > 0) {Analyseur_Syntaxique(); Traite_la_consigne();}
  //-------------------- Traitement des actions de la boucle de base ---------------
  //-------- Faire clignoter la LED de la boucle de base active --------
  LSB = 100; if (Stabilisation_gyroscopique) LSB = 30;
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + LSB; // Réarme pour 0,1S ou 0,03S.
      digitalWrite(LED_Loop_active,!digitalRead(LED_Loop_active));} // Inverser l'état de la LED.
  //----------------- Traite les fonctions analogiques ----------------- 
  if (Phares_actifs) analogWrite(Pilotage_eclairage, Niveau_eclairage_Phares);
  else analogWrite(Pilotage_eclairage, 0);
  if (LASER_actif) analogWrite(LASER, Niveau_eclairage_LASER); // Provisoire @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  else analogWrite(LASER, 0); // Provisoire @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  //--------------- Fin du traitement des actions de la boucle de base -------------
  if (Repondre) Accuse_reception();} // ACR si une consigne a été reçue.
      
void Accuse_reception() {      
  delay(100); Repondre = false; Serial.print(Texte_ACR); Serial.print('*');}
  
void Traite_la_consigne() {
   Texte_ACR = "OK";
   switch (NOMBRE) {
      case 24 : {Texte_ACR = Version_PGM; break;} // S16 Indique la version du programme.   
      case 25 : {digitalWrite(Disjoncteur,LOW); break;} // S1 Disjoncte les LEDs.
      case 26 : {digitalWrite(Disjoncteur,HIGH); break; } // S2 Rétablit les LEDs.
      case 27 : {Phares_actifs = true; digitalWrite(Pilotage_eclairage,HIGH); break;} // S3 Allume les Phares.
      case 28 : {Phares_actifs = false; digitalWrite(Pilotage_eclairage,LOW); break;} // S4 Éteind les Phares.
      case 29 : {LASER_actif = true; digitalWrite(LASER,HIGH); break;} // S5 Allume le LASER.
      case 30 : {LASER_actif = false; digitalWrite(LASER,LOW); break;} // S6 Éteind le LASER.
      case 31 : {Moteurs_ON = false; digitalWrite(LED_Moteurs_OFF,HIGH); break;} // S7 Figer les moteurs.
      case 32 : {Moteurs_ON = true; digitalWrite(LED_Moteurs_OFF,LOW); break;} // S8 Rétablir les moteurs.
      case 33 : {Rapide = true; Texte_ACR = "> C33<" ; break;} // S9 Mvt de coordination rapides.
      case 34 : {Rapide = false; Texte_ACR = "> C34<" ; break;} // S10 Mvt de coordination lents.
      case 35 : {Stabilisation_gyroscopique = true; break;} // S11 Début de stabilisation gyroscopique.
      case 36 : {Stabilisation_gyroscopique = false; break;} // S12 Fin de stabilisation gyroscopique.
      case 37 : {Apprentissage = true; Texte_ACR = "> C37<" ; break;} // Débuter un apprentissage. LED sur S14 @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      case 38 : {Apprentissage = false; Texte_ACR = "> C38<" ; break;} // Terminer une apprentissage.  LED sur S14 @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      case 39 : {Torsion_Active = false; digitalWrite(LED_Torsion_active,LOW); Texte_ACR = "> C39<" ; break;} // S15 Terminer un mouvement de torsion. @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      }
   }
  
void Analyseur_Syntaxique() {
  num_ERR = 0;
  // Extraction du nombre jusqu'à trouver un caractère non chiffre.
  // L'extraction commence au début de Chaine_Memorisee.
  NOMBRE = Chaine_Memorisee.toInt();
  Chaine_Memorisee = "";} // Vider le tampon pour une nouvelle capture.  
  
void Attendre_une_chaine_sur_le_Maitre() {// Cette procédure attend une chaine sur la ligne USB.
//-------------- Définition des broches de sorties Binaires -----------------
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres = 0;
  while (!Sentinelle_detectee) {
    if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
    // Puis concaténer ce caractère y compris la sentinelle :
    if (Compteur_caracteres < LGR_chaine) 
       {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}
    // Si sentinelle détectée, informer la boucle de réception :
    Sentinelle_detectee = (Caractere == '*');}}
  Repondre = true;}
  
 void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}

