/* Programme Démonstrateur directement issu de la version E pour mettre au point sur le
   PUPITRE l'utilisation de la centrale inertielle.
   Programme à associer avec P22F_Demonstrateur_Sonde.ino
   
   UTILISATION DU PROGRAMME :
 ----------------------------
 1) Établir la liaison TX/RX avec l'inverseur,
 2) Faire un RESET sur la Raquette,
 3) Cliquer sur une touche pour passer à l'exploration du clavier.
 4) Cliquer à convenance sur toutes les touches du clavier.
    Si ERR ou version du PGM un BIP est déclenché.
 5) Cliquer sur le bouton central du codeur incrémental se contente de générer un BIP.
 6) Tourner à convenance le codeur incrémental.
 La LED rouge A0 sur la sonde : Rapide allume et Lent éteint la LED.
   
 Ce que fait ce démonstrateur :
 1) Il met en place l'exploitation de la centrale inertielle :
    * Initialisation et Vérification de communication.
    * Gère le Calage du Gyroscope.
    * Fait afficher Le Cap actuel, le Tangage et le roulis.
    * Fait afficher l'écart de route.
 2) Consigne pour sauvegarder la posture actuelle.
 3) Consigne QUITTER.
                                                             PGM            RAM dynamique 
Démonstrateur sans modification :                      14364 Octets (46%)   352 Octets (17%)
Vérification de communication et calage gyroscope :    14718 Octets (47%)   352 Octets (17%)  +354 octets
Affichage du Cap actuel, du tangage et du Roulis :     14838 Octets (48%)   352 Octets (17%)  +120 octets
Affiche l'écart de route et optimisation diverses :    14684 Octets (47%)   352 Octets (17%)  -154 octets
Passage de quelques textes dans l'EEPROM de la Sonde : 14536 Octets (47%)   348 Octets (16%)  -148 octets
Sauvegarder la posture actuelle et consigne QUITTER :  14602 Octets (47%)   348 Octets (16%)   +66 octets

//--------------------- Déclarations pour le matériel ----------------------- */
#include <EEPROM.h>
#include <Adafruit_ssd1306syp.h> // Afficher graphique OLED.
// Déclarations pour le capteur codeur rotatif KY-40 :
#define Codeur_A 2    // D2 : DT
#define Codeur_B 3    // D3 : CLK
// Définition des broches pour la ligne I2C.
#define broche_SDA 11 // D11 utilisée en sortie.
#define broche_SCL 12 // D12 utilisée en sortie.
// Initialisation des broches en sorties et affectations.
Adafruit_ssd1306syp display(broche_SDA, broche_SCL);
//--------------- Définition des broches d'Entrées Analogiques --------------
#define Mesure_U_clavier 6 // Entrée analogique A6 utilisée pour le clavier.
#define BP_central 21 // A7 pour lire le bouton central.
//------------------ Sorties sur broches "Analogiques" ----------------------
#define LED_Clavier 19 // Entrée analogique A3 utilisée comme Sortie Binaire.
//-------------- Définition des broches de sorties Binaires -----------------
#define LED_Erreur 4 // LED rouge : MOUVEMENTS ou ERREUR.
#define BUZZER 6
#define C1 7  // Colonne C1 sur la sortie D7.
#define C2 8  // Colonne C1 sur la sortie D8.
#define C3 9  // Colonne C1 sur la sortie D9.
#define C4 10 // Colonne C1 sur la sortie D10.
#define LED_Loop_active 13 // D13 utilisée pour la LED clignotante "Boucle de base".
//------------------------ Constantes du programme --------------------------
#define Version_PGM "F 07-02-18" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LGR_chaine 20 //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//------------------------- Variables du programme --------------------------
// Note : Les déclarations sont classées par type et par ordre alphabétique croissant.
boolean Gradateur;
boolean LED_Clavier_Clignote;
boolean LASER;
boolean PHARES;
boolean Sens_positif; // Sens de rotation.
boolean Sentinelle_detectee; // Délimiteur de fin détecté.
boolean Verifier_OK; // Vérifier que l'ACR est bien "OK*".

byte BP; // Bouton du claviar activé.
byte BIT_Option;
byte Code_Consigne;
byte Compteur_caracteres;
byte Etat_B;
byte Mode_en_cours; // 1 : EXPLOITER.   2 : Imposer des POSTURES. (Correspondent au BP du clavier.)
                    // 3 : OPTIONS de la sonde.   4 : DONNEES de la sonde.    5: MOUVOIR.
byte NB_a_incrementer_decrementer;                    
byte NB_Repetition_Mvt;
byte Num_des_donnees;
byte Num_Exploitation;
byte Num_des_options;
byte Num_des_postures;
byte RETOUR; // Code de retour d'une fonction à risque. (Code 0 : pas de retour nécessaire.)
byte Transition;
char Caractere;

int CNA; // Mesure analogique retournée par le CNA.
byte PTR; // Pour le pointage en EEPROM.

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

void setup()  {
  //------------------------ Initialisations pour le matériel --------------------------
  display.initialize(); delay(300); display.clear();
  Serial.begin(115200);
  //------------------------------ Affiche écran initial -------------------------------
  display.setTextSize(1); // Taille la plus petite.
  Affiche_cadre_du_haut(11); Aff_TEXTE_EEPROM(11,8); display.print(Version_PGM);
  CADRE(); display.setTextSize(2); // Taille double.
  display.setCursor(38,24); Aff_TEXTE_EEPROM(0,5);
  display.setCursor(32,41); Aff_TEXTE_EEPROM(5,6);
  display.update(); display.setTextSize(1); // Taille la plus petite.
  //-------------------------- Initialisations des booléens ----------------------------
  PHARES = false; LASER = false; Gradateur = false; RETOUR = 0;
  //------------------------- Initialisations des variables ----------------------------
  NB_Repetition_Mvt = 1; 
  //------------------------------ Initialisations des E/S -----------------------------
  pinMode(C1,OUTPUT); pinMode(C2,OUTPUT); pinMode(C3,OUTPUT); pinMode(C4,OUTPUT);
  digitalWrite(C1,LOW); digitalWrite(C2,LOW); digitalWrite(C3,LOW); digitalWrite(C4,LOW); 
  pinMode(LED_Erreur,OUTPUT); digitalWrite(LED_Erreur,LOW);
  pinMode(LED_Clavier,OUTPUT); digitalWrite(LED_Clavier,LOW);
  pinMode(Codeur_A, INPUT); pinMode(Codeur_B, INPUT);
  //------------------------------------------------------------------------------------
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  ATTENDRE_un_BP();  //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  Mode_en_cours = 4; //MENU pour interroger la sonde sur ses donnés capteur, états booléens ...)
  Num_des_donnees = 1; Affiche_Ecran_MENU();  
  Transition = LOW;} // LOW initial simule une rotation pour déclencher le premier menu.

void loop() {
  //------------- Faire clignoter la LED "boucle de base" et Mouvements ----------------
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET) {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + 100; // Réarme pour 0,1S ou 0,03S.
      digitalWrite(LED_Loop_active,!digitalRead(LED_Loop_active)); // Inverser l'état de la LED.
      if (LED_Clavier_Clignote) Inverse_LED_Clavier(); else digitalWrite(LED_Clavier,LOW);
      if (Mode_en_cours == 5) digitalWrite(LED_Erreur,!digitalRead(LED_Erreur));
      else digitalWrite(LED_Erreur,LOW);}
  //------------------------------- Traiter le clavier ---------------------------------
  Tester_le_clavier();
  if (BP > 0) {
      if (BP < 6) {Mode_en_cours = BP; Sens_positif = true; // Pour ouvrir sur la bonne page écran.
                   Num_des_donnees = 1; Num_des_options = 5; Num_Exploitation = 7; Num_des_postures = 9;
                   Affiche_Ecran_MENU(); Traite_Codeur_incremental();} // Boutons à validité permanente.
      if (BP == 7) Traite_la_touche_OUI();
      if (BP == 8) Traite_la_touche_NON();
      if (BP == 13) {PHARES = !PHARES; if (PHARES) Envoyer_consigne(27); else Envoyer_consigne(28);}
      if (BP == 15) {LASER = !LASER; if (LASER) Envoyer_consigne(29); else Envoyer_consigne(30);}
      if (Mode_en_cours == 5) {// Mode MOUVOIR
          for (byte I = NB_Repetition_Mvt; I>0; I--)
              switch (BP) { // Traite les touches de déplacement.
                case 6  : {Envoyer_consigne(7); break;} // Translation à droite.
                case 9  : {Envoyer_consigne(3); break;} // Avancer.
                case 10 : {Envoyer_consigne(14); break;} // Libérer les efforts.
                case 11 : {Envoyer_consigne(4); break;} // Reculer.
                case 12 : {Envoyer_consigne(5); break;} // Tourner à droite.
                case 14 : {Envoyer_consigne(8); break;} // Translater à gauche.
                case 16 :  Envoyer_consigne(6);}}}       // Tourner à gauche.
  //----------------------------- Traite le B.P. central -------------------------------
  if (analogRead(BP_central) < 512) { // Si le B.P. central est activé :
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     BIP();}
  //------------------------------ Traite la rotation ----------------------------------
  Etat_B = digitalRead(Codeur_B);  
  if ((Transition == LOW) && (Etat_B == HIGH)) {    
       if (digitalRead(Codeur_A) == LOW) Sens_positif = false;
       else Sens_positif = true; Afficher_Rot_ou_CLV();
       if (Mode_en_cours == 5) {// Menu des EXPLOITER.
           NB_a_incrementer_decrementer = NB_Repetition_Mvt; Incrementer_Decrementer(9);
           NB_Repetition_Mvt = NB_a_incrementer_decrementer;
       Afficher_le_NB_fois();}
       else {if (Gradateur) Traite_Gradateur(); else Traite_Codeur_incremental();}}
  Transition = Etat_B;}
  
void Incrementer_Decrementer(byte MAXI) {// La valeur minimale est toujours 1.
  if (Sens_positif) {NB_a_incrementer_decrementer++; 
                     if (NB_a_incrementer_decrementer > MAXI) NB_a_incrementer_decrementer = 1;}
  else { if (NB_a_incrementer_decrementer > 1) NB_a_incrementer_decrementer--;
         else NB_a_incrementer_decrementer = MAXI;}}
  
void Traite_la_touche_OUI() {
   if (Mode_en_cours == 1) { // Menu EXPLOITER.
      switch (Num_Exploitation) {
         case 1 : {Envoyer_consigne(18); break;} // Enregistrer en EEPROM la posture actuelle.
         case 2 : {Envoyer_consigne(54); break;} // LASER et Phares forcés à 127.
         // Signale à la sonde que le gradateur agit sur les phares :
         case 3 : {Envoyer_consigne(43); Gradateur = true; AFFICHE_puissance_eclairages(); break;}
         // Signale à la sonde que le gradateur agit sur le LASER.
         case 4 : {Envoyer_consigne(42); Gradateur = true; AFFICHE_puissance_eclairages(); break;}
         case 5 : {Envoyer_consigne(55); break;} // Réveiller la sonde. 
         case 6 : {Envoyer_consigne(59); Affiche_Calage_Gyroscope(1,37); break;} // Calage du gyroscope.       
         case 7 : Envoyer_consigne(58); // Test de communication MPU6050.
                  if (Chaine_Memorisee == "OK") display.print(Chaine_Memorisee);
         case 8 : {Envoyer_consigne(61);break;}}
      display.update();}
   if (Mode_en_cours == 2)  {// Menu des POSTURES. 
       switch (Num_des_postures) {
         case 1 : {Envoyer_consigne(1); break;}  // VEILLE.
         case 2 : {Envoyer_consigne(16); RETOUR = 17; break;} // Stable Raisonnable.
         case 3 : {Envoyer_consigne(10); RETOUR = 11; break;} // Hauteur Maximale.
         case 4 : {Envoyer_consigne(15); break;} // Moteurs au Neutre opérationnel.
         case 5 : {Envoyer_consigne(19); break;} // Restituer la posture apprise en EEPROM.
         case 6 : {Envoyer_consigne(23); break;}  // Configuration décollage.
         case 7 : {Envoyer_consigne(22); break;} // Atterrissage.
         case 8 : {Envoyer_consigne(21); break;} // Déposer la Sonde.
         case 9 : Envoyer_consigne(2);}       // Stable Transversal.
   display.update();}    
   if (Mode_en_cours == 3) { // Menu des OPTIONS.
       switch (Num_des_options) { // Change l'étét du booléen système.
         case 1 : {Envoyer_consigne(25); break;} // Mode sommeil.
         case 2 : {Envoyer_consigne(32); break;} // Moteurs ON.
         case 3 : {Envoyer_consigne(33); break;} // Mouvements rapides.
         case 4 : {Envoyer_consigne(35); break;} // Stabilisation Gyroscopique.
         case 5 :  Envoyer_consigne(37);} // Mode apprentissage.
       Affiche_etat_option_en_cours();}}  // Display.update();
 
void Traite_la_touche_NON() {
  if (RETOUR > 0) Envoyer_consigne(RETOUR); // Configuration de retour des fonctions à risque.
  if (Mode_en_cours == 1) { // Menu EXPLOITATION. (Signale la fin du gradateur.)
      if ((Num_Exploitation == 3) || (Num_Exploitation == 4)) //Gradateur Phares ou Gradateur LASER.
          {Envoyer_consigne(53); Effacer_zone(50, 52, 3); Gradateur = false; display.update();}}         
  if (Mode_en_cours == 3) { // Menu des OPTIONS.
      switch (Num_des_options) { // Change l'étét du booléen système.
        case 1 : {Envoyer_consigne(26); break;} // Mode sommeil.
        case 2 : {Envoyer_consigne(31); break;} // Moteurs ON.
        case 3 : {Envoyer_consigne(34); break;} // Mouvements rapides.
        case 4 : {Envoyer_consigne(36); break;} // Stabilisation Gyroscopique.
        case 5 :  Envoyer_consigne(38);} // Fin du Mode apprentissage.
        Affiche_etat_option_en_cours();}} // Terminée par display.update();             
  
void Traite_Codeur_incremental() {
   LED_Clavier_Clignote = false;
   switch (Mode_en_cours) {
     case 1 : {MENU_EXPLOITER(); break;}
     case 5 : {MENU_MOUVOIR(); LED_Clavier_Clignote = true; break;}
     case 2 : {MENU_POSTURES(); break;}
     case 3 : {MENU_des_OPTIONS(); LED_Clavier_Clignote = true; break;}
     case 4 :  MENU_des_DONNEES();}}
  
void Afficher_le_NB_fois() {
  Effacer_zone(66,55,1); display.setCursor(18,55); Aff_TEXTE_EEPROM(89,8);
  display.print(NB_Repetition_Mvt); Aff_TEXTE_EEPROM(634,5); display.update();}
     
void MENU_MOUVOIR(){
   display.setCursor(30,20); Aff_TEXTE_EEPROM(593,11); Prepare_afficher_item();
   Aff_TEXTE_EEPROM(96,18); display.update();}
   
void MENU_POSTURES(){
  NB_a_incrementer_decrementer = Num_des_postures; Incrementer_Decrementer(9);
  Num_des_postures = NB_a_incrementer_decrementer;
  Affiche_Ecran_POSTURES(); display.update();}
  
void Affiche_Ecran_POSTURES(){
  display.setCursor(45,21); Aff_TEXTE_EEPROM(36,7); Effacer_zone(6, 55, 17);
  Prepare_afficher_item();
  switch (Num_des_postures) {
    case 1 : {Aff_TEXTE_EEPROM(463,6); break;}  // VEILLE.
    case 2 : {Aff_TEXTE_EEPROM(469,18); AFFICHE_RETOUR(); break;} // Stable Raisonnable.
    case 3 : {Aff_TEXTE_EEPROM(487,16); AFFICHE_RETOUR(); break;} // Hauteur Maximale.
    case 4 : {Aff_TEXTE_EEPROM(503,20); break;} // Moteurs au Neutre opérationnel.
    case 5 : {Aff_TEXTE_EEPROM(523,17); break;} // Restituer la posture apprise en EEPROM.
    case 6 : {BIP(); Aff_TEXTE_EEPROM(552,9); Aff_TEXTE_EEPROM(619,12); break;}  // Décollage.
    case 7 : {Aff_TEXTE_EEPROM(540,12); break;} // Atterrissage.
    case 8 : {Aff_TEXTE_EEPROM(561,14); break;} // Déposer la Sonde.
    case 9 : {Aff_TEXTE_EEPROM(575,18);}}       // Stable Transversal.
    if (Num_des_postures != 6) display.print('.');}
    
void AFFICHE_RETOUR() {display.setCursor(6,55); display.print(char(16)); Aff_TEXTE_EEPROM(604,15); BIP();} 
     
void Traite_Gradateur() {
  if (Sens_positif) Envoyer_consigne(40); else Envoyer_consigne(41); AFFICHE_puissance_eclairages();} 
      
void AFFICHE_puissance_eclairages() {
  if (Num_Exploitation == 3) Demande_Information(46); else Demande_Information(47); // Demande la puissance.
  Effacer_zone(50, 52, 3); display.setCursor(50,52); display.print(Chaine_Memorisee); display.update();} 
     
void MENU_EXPLOITER(){
  NB_a_incrementer_decrementer = Num_Exploitation; Incrementer_Decrementer(8);
  Num_Exploitation = NB_a_incrementer_decrementer;
  Affiche_Ecran_EXPLOITER(); display.update();}
  
void Prepare_afficher_item() {Effacer_zone(1, 37, 21); display.setCursor(1,37);} 
  
void Affiche_Ecran_EXPLOITER(){
  Prepare_afficher_item();
  switch (Num_Exploitation) {
    case 1 : {Aff_TEXTE_EEPROM(248,19); break;} // Sauvegarder la posture actuelle.
    case 2 : {Aff_TEXTE_EEPROM(443,20); break;} // Phare et LASER = 127.
    case 3 : {Aff_TEXTE_EEPROM(420,17); break;} // Gradateur Phare.
    case 4 : {Aff_TEXTE_EEPROM(420,10); Aff_TEXTE_EEPROM(437,6); break;} // Gradateur LASER.
    case 5 : {Aff_TEXTE_EEPROM(410,10); Aff_TEXTE_EEPROM(5,6); break;} // Réveiller la sonde.
    case 6 : {Affiche_Calage_Gyroscope(1,37); break;}  // Calage du gyroscope.
    case 7 : {Aff_TEXTE_EEPROM(406,4); Aff_TEXTE_EEPROM(166,11); break;} // Test communication avec le MPU6050.
    case 8 : {Aff_TEXTE_EEPROM(267,7); ESP_Point_Int();}}}
    
void ESP_Point_Int() {Aff_TEXTE_EEPROM(199,2);}
       
void MENU_des_DONNEES(){
  NB_a_incrementer_decrementer = Num_des_donnees; Incrementer_Decrementer(5);
  Num_des_donnees = NB_a_incrementer_decrementer;
  Effacer_zone(7, 19, 19); Effacer_zone(5, 32, 20); Effacer_zone(7, 42, 19); Effacer_zone(7, 52, 19);
  CADRE(); display.drawLine(1, 28, 126, 28, WHITE);
  Affiche_Ecran_des_DONNEES(); display.update();}
 
void Affiche_Ecran_des_DONNEES(){
  switch (Num_des_donnees) {
              // Ci dessous : Page Phares et LASER.
    case 1 : {display.setCursor(15,19); Aff_TEXTE_EEPROM(361,17);
              display.setCursor(19,32); Aff_TEXTE_EEPROM(378,15);
              Demande_Information(44); // Retourne t'état des booléens système.
              display.setCursor(25,42); OUI_NON(5); display.setCursor(85,42); OUI_NON(6);
              Demande_Information(46); display.setCursor(25,52); display.print(Chaine_Memorisee);
              Demande_Information(47); display.setCursor(85,52); display.print(Chaine_Memorisee); break;}
               // Ci-dessous Page Version / SOMMEIL / Moteurs / Mvt Rapides.
    case 2 : {Demande_Information(24);  // version du PGM de la Sonde.
              display.setCursor(7,19); Aff_TEXTE_EEPROM(201,8); display.print(Chaine_Memorisee);
              Demande_Information(44); // Retourne t'état des booléens système.
              display.setCursor(7,32); Affiche_un_Soleil(); Aff_TEXTE_EEPROM(208,11); OUI_NON(0);                                          
              display.setCursor(7,42); Aff_TEXTE_EEPROM(219,15); OUI_NON(1);
              display.setCursor(7,52); Aff_TEXTE_EEPROM(234,14); OUI_NON(2); break;}
              // Ci dessous : Page Météo et Télémètre.
    case 3 : {display.setCursor(7,19); Demande_Information(51); display.print(Chaine_Memorisee);
              display.setCursor(7,32); Demande_Information(48); // Retourne la valeur de la Température.              
              display.print(Chaine_Memorisee); Affiche_Degre(); display.print('C');
              Demande_Information(49); // Retourne la valeur de l'hygrométrie relative.
              display.setCursor(7,42); display.print(Chaine_Memorisee); Demande_Information(50);
              display.setCursor(7,52); display.print(Chaine_Memorisee); break;}
               // Ci-dessous : Page Distance, Stabilisation / Bouclier / Apprentissage.
    case 4 : {Demande_Information(45);
              display.setCursor(7,19); display.print(Chaine_Memorisee); Demande_Information(44);
              display.setCursor(7,32); Aff_TEXTE_EEPROM(161,16); OUI_NON(3);                                          
              display.setCursor(7,42); Aff_TEXTE_EEPROM(274,16); OUI_NON(4);
              display.setCursor(7,52); Aff_TEXTE_EEPROM(290,16); OUI_NON(7); break;}
              // Ci dessous : Les données de navigation, CAP, Tangage, Roulis, Ecart de route.
    case 5 : {display.setCursor(7,19); Demande_Information(52); // Affiche le Cap actuel.
              display.print(Chaine_Memorisee);Affiche_Degre();
         Demande_Information(56);      
        display.setCursor(5,32); display.print(Chaine_Memorisee); // Affiche Tangage et Roulis.
              Affiche_Calage_Gyroscope(7,42);
            
              display.setCursor(7,52);
              Demande_Information(60); display.print(Chaine_Memorisee); Affiche_Degre(); break;}}}
     
void Affiche_Degre() {display.print(char(247));}

void Affiche_un_Soleil() {display.print(char(15));}
      
void Affiche_Calage_Gyroscope(byte X, byte Y) {
  Effacer_zone(X+79,Y-1,4); display.setCursor(X,Y); Demande_Information(57);
  display.print(Chaine_Memorisee); Affiche_Degre();}
 
void OUI_NON(byte Position) {
  if (Chaine_Memorisee.charAt(Position) == '1') Aff_TEXTE_EEPROM(355,3); else Aff_TEXTE_EEPROM(358,3);} 
        
void MENU_des_OPTIONS() {
  NB_a_incrementer_decrementer = Num_des_options; Incrementer_Decrementer(5);
  Num_des_options = NB_a_incrementer_decrementer; Effacer_zone(5, 30, 20); display.setCursor(5,30);
  switch (Num_des_options) {
    case 1 : {Aff_TEXTE_EEPROM(114,14); display.setCursor(72,30);
              Affiche_un_Soleil(); break;}  // Mode "couper les éclairages".
    case 2 : {Aff_TEXTE_EEPROM(128,13); break;}  // Moteurs OFF.
    case 3 : {Aff_TEXTE_EEPROM(141,20); break;}  // Mouvements rapides.
    case 4 : {Aff_TEXTE_EEPROM(161,17); break;}  // Stabilisation Gyroscopique.
    case 5 : {Aff_TEXTE_EEPROM(181,20);}}  // Mode apprentissage.
  BIT_Option = Num_des_options; BIT_Option--; if (BIT_Option == 4) BIT_Option = 7;
  Affiche_etat_option_en_cours();}
  
void Affiche_etat_option_en_cours() {
  Demande_Information(44); // Etat des options.
  Effacer_zone(56, 48, 4); display.setCursor(50,48); display.print('(');
  OUI_NON(BIT_Option); display.print(')'); display.update();}
   
void Affiche_Ecran_MENU() {
  Gradateur = false;
  display.clear(); CADRE_UP(); display.drawLine(100, 0, 100, 15, WHITE); display.setCursor(10,4);
  switch (Mode_en_cours) {
    case 1 : {Aff_TEXTE_EEPROM(75,14); break;}
    case 2 : {Aff_TEXTE_EEPROM(33,14); break;}
    case 3 : {Aff_TEXTE_EEPROM(47,14); break;}
    case 4 : {Aff_TEXTE_EEPROM(61,14); break;}
    case 5 :  Aff_TEXTE_EEPROM(19,14); Afficher_le_NB_fois();}
  Afficher_Rot_ou_CLV();} // Terminé par display.update().
    
void Afficher_Rot_ou_CLV() {
  Effacer_zone(106,4,3); display.setCursor(106,4);
  if (Mode_en_cours == 5) Aff_TEXTE_EEPROM(178,3); else Aff_TEXTE_EEPROM(631,3); display.update();}
  
//---------------------------------- Diverses procédures de service --------------------------------
void Demande_Information(byte Code_consigne) {
  Verifier_OK = false; Transmettre_le_code_consigne(Code_consigne);}

void Envoyer_consigne(byte Code_consigne) {
  Verifier_OK = true; Transmettre_le_code_consigne(Code_consigne);}

void Transmettre_le_code_consigne(byte Code_consigne) {  
  Serial.print(Code_consigne); Serial.print('*'); Attendre_une_chaine_sur_le_Maitre();
  if ((Chaine_Memorisee != "OK") && (Verifier_OK)) {
      BIP(); Effacer_zone(106, 4, 3); display.setCursor(106,4);
      display.print(Chaine_Memorisee); display.update();}}

//--------------------------------- Gestion du clavier à 16 touches --------------------------------
void Tester_le_clavier() {
   BP = 0; // Indique une non utilisation du clavier.
  Teste_Colonne(C1); Teste_Colonne(C2); Teste_Colonne(C3); Teste_Colonne(C4);}

void Teste_Colonne(byte Colonne) {
  digitalWrite(Colonne, HIGH); // Passe la colonne au +5Vcc.
  CNA = analogRead(Mesure_U_clavier);
  if (CNA > 90) // Codage du BP sollicité.
     {BP = Colonne + 6; if (CNA > 286) BP = BP - 4; if (CNA > 511) BP = BP - 4; if (CNA > 718) BP = BP - 4;
      while (analogRead(Mesure_U_clavier) > 90) delay (5); delay (10);} // Attendre le relâcher.
  digitalWrite(Colonne, LOW);} // Repasse la colonne à zéro volts.
  
void ATTENDRE_un_BP() {
    while (BP == 0) {
           Tester_le_clavier(); Inverse_LED_Clavier(); delay(50);}}
           
void Inverse_LED_Clavier() {digitalWrite(LED_Clavier,!digitalRead(LED_Clavier));}

//--------------------------------------------------------------------------------------------------
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); digitalWrite(LED_Erreur,HIGH);
  delay(100); noTone(BUZZER); digitalWrite(LED_Erreur,LOW); pinMode(BUZZER, INPUT);}

void Attendre_une_chaine_sur_le_Maitre() {// Cette procédure attend une chaine sur la ligne USB.
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres = 0;
  while (!Sentinelle_detectee) {
    if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
    Sentinelle_detectee = (Caractere == '*'); // Si sentinelle détectée, informer la boucle de réception.
    // Puis concaténer ce caractère SAUF la sentinelle :
    if ((Compteur_caracteres < LGR_chaine) && (!Sentinelle_detectee)) 
       {Chaine_Memorisee = Chaine_Memorisee + Caractere; Compteur_caracteres++;}}}}    
         
void Envoyer_Consigne(byte CODE) {Serial.print(CODE); Serial.print('*');}

//--------------------------------- Routines pour les affichages -----------------------------------
void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {
  for (byte I=0; I<Longueur; I++) {display.print(char(EEPROM.read(PTR))); PTR++;}; }
  
void Effacer_zone(byte X, byte Y, byte NB_caracteres) {
  byte I = 10; 
  for (byte J=0; J < I; J++)
       display.drawLine(X, Y + J, X + (NB_caracteres * 6), Y + J, BLACK);}

void CADRE_UP() {display.drawRect(0, 0, 128, 16, WHITE);}
void CADRE() {display.drawRect(0, 16, 128, 48, WHITE);}
void Affiche_cadre_du_haut(byte X) {CADRE_UP(); display.setCursor(X,4);}

